﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Interop;
using Gw2Api;
using System.ComponentModel;
using Gw2WvWMapControl;
using System.Diagnostics;

namespace Gw2Overlay
{
    /// <summary>
    /// Interaction logic for GraphWindow.xaml
    /// </summary>
    public partial class GraphWindow : OverlayWindow
    {
        public WvWApi WvWApiInstance;
        private OverlordWindow _ow;
        public AudioNotification audio;

        public GraphWindow(OverlordWindow ow)
        {
            InitializeComponent();
            this._ow = ow;
            this.audio = AudioNotification.None;
            MouseLeftButtonDown += delegate { wvWMapControl1.ShowStatusTooltip = false; };
            MouseLeftButtonUp += delegate { wvWMapControl1.ShowStatusTooltip = true; _ow.UpdateInfo(this); };
            WvWApiInstance = Gw2ApiFactory.WvWApi;
            wvWMapControl1.WvWApi = WvWApiInstance;
            wvWMapControl1.SliceUpdate += new WvWMapControl.SliceUpdateHandler(OnSliceUpdate);
            CreateContextMenu();
            Debug.WriteLine("WvWControl Width: " + wvWMapControl1.ActualWidth + ", Height:" + wvWMapControl1.ActualHeight);
        }

        protected override void OnInteractable(bool interactable)
        {
            if (interactable)
            {
                backrect.Visibility = System.Windows.Visibility.Visible;
                this.ResizeMode = ResizeMode.CanResizeWithGrip;
            }
            else
            {
                backrect.Visibility = System.Windows.Visibility.Hidden;
                this.ResizeMode = ResizeMode.NoResize;
            }
        }

        public void CreateContextMenu()
        {
            MenuItem matchups = this.Resources["Matchups"] as MenuItem;
            MenuItem matchupbyworld = this.Resources["MatchupByWorld"] as MenuItem;
            
            WorldNameApi names = Gw2ApiFactory.WorldNameApi;
            foreach (var match in WvWApiInstance.GetMatchups())
            {
                MenuItem mi = new MenuItem();
                mi.Header = match.ToString(names);
                mi.Click += new RoutedEventHandler(MatchClick);
                mi.Tag = match.wvw_match_id;
                matchups.Items.Add(mi);
            }

            foreach (var name in names.GetWorldNames())
            {
                MenuItem mi = new MenuItem();
                mi.Header = name;
                mi.Click += new RoutedEventHandler(MatchClick);
                var mu = WvWApiInstance.GetMatchupByWorldName(name);
                if (null != mu)
                {
                    mi.Tag = mu.wvw_match_id;
                    matchupbyworld.Items.Add(mi);
                }
            }
        }

        private void OnSliceUpdate(object sender, WvWMapControl.SliceUpdateEventArgs e)
        {
            var diff = e.diff;
            switch (audio)
            {
            case AudioNotification.Voice:
                {
                    var objective = WvWApi.GetObjectivePropertyById(diff.objectiveId);
                    if (null != objective && objective.point > 0)
                    {
                        string s = diff.to + " has taken " + objective.name;
                        TtsService.Instance.Speak(s);
                    }
                }
                break;
            }
        }

        private void NewWindowClick(object sender, RoutedEventArgs e)
        {
            _ow.CreateNewGraphWindow(this.wvWMapControl1.WvWMap, this.wvWMapControl1.WvWMatchup, 0, 0);
        }

        private void CloseClick(object sender, RoutedEventArgs e)
        {
            _ow.CloseWindow(this);
            this.Close();
        }

        private void QuitClick(object sender, RoutedEventArgs e)
        {
            Debug.Flush();
            _ow.Shutdown();
        }

        private void SoundClick(object sender, RoutedEventArgs e)
        {
            MenuItem mi = (MenuItem)sender;
            string sound = mi.Tag.ToString();
            AudioNotification au = AudioNotification.None;
            if (Enum.TryParse<AudioNotification>(sound, false, out au))
            {
                this.audio = au;
            }
        }

        private void MapClick(object sender, RoutedEventArgs e)
        {
            MenuItem mi = (MenuItem)sender;
            string map = mi.Name;
            try
            {
                this.wvWMapControl1.PositionApi = null;
                this.wvWMapControl1.WvWMap = (WvWMaps)Enum.Parse(typeof(WvWMaps), map, true);
            }
            catch (Exception)
            {
                this.wvWMapControl1.PositionApi = Gw2ApiFactory.PositionApi;
            }
            _ow.UpdateInfo(this);
        }

        private void MatchClick(object sender, RoutedEventArgs e)
        {
            MenuItem mi = (MenuItem)sender;
            string id = mi.Tag.ToString();
            this.wvWMapControl1.WvWMatchup = id;
            _ow.UpdateInfo(this);
        }

        private void ResizeClick(object sender, RoutedEventArgs e)
        {
            MenuItem mi = (MenuItem)sender;
            GraphWindowSizes size = (GraphWindowSizes)Enum.Parse(typeof(GraphWindowSizes), mi.Tag.ToString(), true);
            this.Width = (int)size;
            this.Height = (int)size;
            _ow.UpdateInfo(this);
        }

        private void TransparentClick(object sender, RoutedEventArgs e)
        {
            MenuItem mi = (MenuItem)sender;
            int value = Convert.ToInt32(mi.Tag.ToString());
            double opactiy = value / 100.0;
            this.Opacity = opactiy;
            _ow.UpdateInfo(this);
        }

        override public void DoHitTest(Point pt)
        {
            try
            {
                Visual target = this.wvWMapControl1.HitTestTarget;
                HitTestResult result = VisualTreeHelper.HitTest(target, target.PointFromScreen(pt));
                if (null != result && result.VisualHit is Shape)
                {
                    this.wvWMapControl1.SetHighlightElement((Shape)result.VisualHit);
                }
                else
                {
                    this.wvWMapControl1.ClearHighlightElement();
                }
            }
            catch (Exception)
            {
            }
        }

        private void WindowSizeChanged(object sender, SizeChangedEventArgs e)
        {
            this.Height = e.NewSize.Width;
            _ow.UpdateInfo(this);
        }

        private void OnWindowClosing(object sender, CancelEventArgs e)
        {
            _ow.CloseWindow(this);
        }
    }


    public enum GraphWindowSizes
    {
        Tiny = 70,
        Small = 120,
        Medium = 170,
        Large = 220,
        Huge = 320,
    }

    public enum AudioNotification
    {
        None,
        Voice,
        Ping,
        Beep,
    }
}
